;; Catalogs.scm
;; Df. du GUI du programme

(module catalogs mzscheme
  (require (lib "class.ss")
           (lib "mred.ss" "mred")
           (lib "file.ss")
           "db.scm"
           "utils.scm"
           "splitter.scm"
           "catsearch.scm"
           "catview.scm")
  
  (define app-name "Catalogs")
  (define app-version '(0 7))
  (define prefs-file (build-path (find-system-path 'home-dir) ".catalogs"))
  (define catalogs-path
    (get-preference 'catalogs-path 
                    (lambda () (dir-slashify (find-system-path 'home-dir)))
                    #f
                    prefs-file))
  (define ui-font-size 
    (get-preference 'ui-font-size 
                    (lambda () 12)
                    #f
                    prefs-file))
  
  (define pp-catalogs
      (get-preference 'pretty-print-catalogs 
                    (lambda () #f)
                    #f
                    prefs-file))
  (define splash
    (let* ((s (let ((ss (instantiate frame% ()
                          (label app-name)
                          (style '(no-resize-border no-system-menu)))))
                (send ss set-label-font 
                      (make-object font% 14 'modern 'normal 'bold #f))
                ss))
           (bmp-dc (instantiate bitmap-dc% 
                     ((make-object bitmap% 
                        (find-executable-path 
                         (find-system-path 'exec-file) 
                         "catsres/catalogs.jpg")
                        'jpeg))))
           (hp (instantiate horizontal-panel% (s) 
                 (alignment '(center center))
                 (horiz-margin 10)
                 (vert-margin 10))) 
           (c (instantiate canvas% (hp)
                (min-width 157)
                (min-height 144)
                (paint-callback
                 (lambda (c dc)
                   (send dc draw-bitmap 
                         (send bmp-dc get-bitmap)
                         0 0)))))
           (p (instantiate vertical-panel% (hp) (alignment '(center center)))))
      (instantiate message% ((format "  ~a  " app-name) p))
      (instantiate message% ((format "  Version ~a.~a  " (car app-version) (cadr app-version)) p))
      (instantiate message% ("(C)M.T." p))
      (send s center)
      s))
  
  (define (exit?)
    (define r #f)
    (let* ((s (let ((ss (instantiate dialog% ()
                          (label app-name))))
                ss))
           (bmp-dc (instantiate bitmap-dc% 
                     ((make-object bitmap% 
                        (find-executable-path 
                         (find-system-path 'exec-file) 
                         "catsres/exclam2.gif")
                        'gif))))
           (hp (instantiate horizontal-panel% (s) 
                 (alignment '(center center))
                 (horiz-margin 10)
                 (vert-margin 10)
                 (spacing 10)
                 )) 
           (c (instantiate canvas% (hp)
                (min-width 50)
                (min-height 50)
                (stretchable-height #f)
                (stretchable-width #f)
                (paint-callback
                 (lambda (c dc)
                   (send dc draw-bitmap 
                         (send bmp-dc get-bitmap)
                         0 0)))))
           (p (instantiate vertical-panel% (hp) (alignment '(center center)))))
      (instantiate message% ((format "You are about to quit ~a" app-name) p))
      (instantiate message% ("Proceed ?" p))
      (let ((v (instantiate horizontal-panel% (p) 
                 (alignment '(center center)))))
        (send (instantiate button% 
                ("Ok" v (lambda (b e) (set! r #t) (send s on-exit)))
                (style '(border))) focus)
        (instantiate button% 
          ("Cancel" v (lambda (b e) (send s on-exit)))))
      (send s center)
      (send s show #t)
      r))
  
  ; la classe de la fentre principale
  (define main-window%
    (class frame%
      (rename (super-on-subwindow-char on-subwindow-char))
      
      (define/override (can-close?)
        (exit?))
      
      (define/override (on-subwindow-char target event)
        (cond
          ((memv (send event get-key-code) '(#\return numpad-enter))
           #f)
          ((send catalogs-panel frame-process-char? target event)
           (super-on-subwindow-char target event))
          (else #f)))
      
      (super-instantiate ())))
  
  ; la fentre principale et ses menus
  (define main-window
    (let* ((f (instantiate main-window% ("Catalogs" #f)
                (width 800) (height 600)
                (x 100) (y 100)
                (spacing 6)))
           (menu-bar 
            (instantiate menu-bar% () (parent f)))
           (file-menu
            (instantiate menu% ()
              (parent menu-bar)
              (label "&File")))
           (command-menu
            (instantiate menu% ()
              (parent menu-bar)
              (label "&Commands")))
           (help-menu
            (instantiate menu% ()
              (parent menu-bar)
              (label "&Help"))))
      
      (define add
        (case-lambda
          ((menu name)
           (if (string=? name "-")
               (instantiate separator-menu-item% () (parent menu)) 
               (instantiate menu-item% ()
                 (label name)
                 (parent menu)
                 (callback void))))
          ((menu name sym)
           (instantiate menu-item% ()
             (label name)
             (parent menu)
             (callback (lambda (x y) (dispatch sym)))))
          ((menu name sym sc)
           (instantiate menu-item% ()
             (label name)
             (parent menu)
             (shortcut sc)
             (callback (lambda (x y) (dispatch sym)))))
          ))
      
      (define (dispatch sym)
        (case sym
          ((hide/show-tools)
           (send tools/catalogs-splitter
                 change-children 
                 (lambda (l)
                   (if (send search-panel is-shown?)
                       `(,catalogs-panel)
                       `(,search-panel ,catalogs-panel)))))
          ((about) 
           (let* ((s (instantiate dialog% ()
                       (parent main-window)
                       (label "About...")))
                  (bmp-dc (instantiate bitmap-dc% 
                            ((make-object bitmap% 
                               (find-executable-path 
                                (find-system-path 'exec-file) 
                                "catsres/catalogs.jpg")
                               'jpeg))))
                  
                  (hp (instantiate horizontal-panel% (s) 
                        (alignment '(center center))
                        (spacing 10)
                        (vert-margin 10)
                        (horiz-margin 10))) 
                  (c (instantiate canvas% (hp)
                       (min-width 157)
                       (min-height 144)
                       (paint-callback
                        (lambda (c dc)
                          (send dc draw-bitmap 
                                (send bmp-dc get-bitmap)
                                0 0)))))
                  
                  (p (instantiate vertical-panel% (hp) 
                       (alignment '(center center)))))
             
             (send p set-label-font 
                   (make-object font% 12 'modern 'normal 'bold #f))
             
             (instantiate message% ((format "  ~a  " app-name) p))
             (instantiate message% ((format "  Version ~a.~a  " (car app-version) (cadr app-version)) p))
             (instantiate message% ("A little DrScheme application" p))
             (instantiate message% ("(C) Martial Tarizzo" p))
             (send (instantiate button% ("Close" p (lambda (b e) (send s on-exit)))
                     (style '(border))) focus)
             (send s center)
             (send s show #t)))
          ((change-cats-folder)
           (let ((r (get-directory 
                     (string-append "Current catalogs folder : " catalogs-path )
                     f #f '())))
             (when r
               (set! catalogs-path (dir-slashify r))
               (send catalogs-panel set-cats-directory catalogs-path)
               (send search-panel set-cats-directory catalogs-path)
               (put-preferences '(catalogs-path) 
                                `(,(dir-slashify r))
                                void 
                                prefs-file))))
          ((exit)
           (when (send main-window can-close?)
             (send main-window on-exit)))
          ))
      
      (add file-menu "&Change catalogs folder ..." 'change-cats-folder #\F)
      (add file-menu "-")
      (add file-menu "Exit" 'exit #\X)
      (add command-menu "Hide/Show tools panel" 'hide/show-tools #\T)
      ;(add help-menu "&Info" 'info #\I)
      (add help-menu "&About" 'about #\A)
      
      (send f create-status-line) 
      f))
  
  ; lments de l'interface  
  (define tools/catalogs-splitter
    (instantiate horizontal-splitter% (main-window)
      (spacing 6)))
  
  (define sp%
    (class search-panel%
      (define search-info-frame 
        (instantiate frame% ("Searching !")
          (parent main-window)
          (width 600)
          (height 100)
          (min-height 0)
          (stretchable-height #t)
          (style '(no-resize-border no-system-menu))))
      
      (define p (instantiate vertical-panel% (search-info-frame) 
                  (alignment '(center center))))
      
      (define m (instantiate message% ()
                  (label "")
                  (parent p)
                  (stretchable-width #t)))
      
      (define b (instantiate button% ()
                  (label "Abort search !")
                  (parent p)
                  (callback
                   (lambda (b c)
                     (send this abort-search)
                     (send search-info-frame on-exit)))))
      
      (define/override (search-message msg)
        (if msg
            (begin
              (when (not (send search-info-frame is-shown?))
                (send* search-info-frame (center) (show #t)))
              (send m set-label msg)
              (send main-window set-status-text msg))
            (send search-info-frame on-exit)))
      
      (define/override (on-search-result-selected l)
        (send catalogs-panel select-data l))
      
      (super-instantiate ())))
  
  (define search-panel
    (let ((sp (instantiate sp% ()
                (parent tools/catalogs-splitter)
                (ui-font-size ui-font-size)
                (style '(border))
                (min-height 250))))
      (send sp set-label-position 'vertical)
      (send tools/catalogs-splitter attach-top sp)
      sp)
    )
  
  ; le panneau des catalogues, liste des rpertoires et fichiers
  (define cv% 
    (class catalog-viewer%
      
      (define/override (on-cat-changed)
        (send main-window set-status-text 
              (send this get-current-cat-name)))
      
      (define/override (on-process-message s)
        (send main-window set-status-text s))
      
      (define/override (on-cat-added/deleted)
        (send search-panel update-cats-display))
      
      (define add-info-frame 
        (instantiate frame% ("Adding catalog !")
          (parent main-window)
          (width 600)
          (height 100)
          (min-height 0)
          (stretchable-height #t)
          (style '(no-resize-border no-system-menu))))
      
      (define p (instantiate vertical-panel% (add-info-frame) 
                  (alignment '(center center))))
      
      (define m (instantiate message% ()
                  (label "")
                  (parent p)
                  (stretchable-width #t)))
      
      (define b (instantiate button% ()
                  (label "Abort !")
                  (parent p)
                  (callback
                   (lambda (b c)
                     (send this abort-add-catalog)
                     (send add-info-frame on-exit)))))
      
      (define/override (on-add-catalog-progress msg)
        (if msg
            (begin
              (when (not (send add-info-frame is-shown?))
                (send* add-info-frame (center) (show #t)))
              (send m set-label msg)
              (send main-window set-status-text msg))
            (send add-info-frame on-exit)))
      
      (super-instantiate ())))
  
  (define catalogs-panel
    (let ((cp (instantiate cv% ()
                (cats-directory catalogs-path)
                (parent tools/catalogs-splitter)
                (ui-font-size ui-font-size)
                (style '(border)))))
      (send tools/catalogs-splitter attach-bottom cp)
      cp))
  
  (set-pretty-print-catalog! pp-catalogs)

  (send main-window center)
  (send main-window maximize #t)
  (send main-window show #t)
  (send splash show #t)
  (send catalogs-panel set-cats-directory catalogs-path)
  (send search-panel set-cats-directory catalogs-path)
  (instantiate timer% () 
    (notify-callback (lambda () (send splash on-exit)))
    (interval 2000)
    (just-once? #t)) 
  
  )
