(module catsearch mzscheme
  (require (lib "class.ss")
           (lib "string.ss")
           (lib "mred.ss" "mred")
           "utils.scm"
           "db.scm"
           "splitter.scm")
  
  (provide search-panel%)
  
  (define file-string-maxlen 50)
  
  (define search-panel%
    (class vertical-splitter%
      ; FIELDS
      (init-field 
       (cats-directory (find-system-path 'home-dir))
       (ui-font-size 12))
      
      ; CALLBACKS
      (define/public (search-message msg) #t)
      
      (define/public (on-search-result-selected l) #t)
      
      ; METHODES
      ; mise  jour de la liste des catalogues
      (define/public (update-cats-display)
        (send search-cats-list clear)
        (send search-results clear)
        (send cats init-catalogs)
        (for-each
         (lambda (c)
           (send search-cats-list append
                 (send c get-name)))
         (send cats get-catalogs)))
      
      (define/public (set-cats-directory s)
        (set! cats-directory (dir-slashify s))
        (set! cats (instantiate catalogs% (cats-directory)))
        (update-cats-display))
      
      (define/public (get-cats-directory) cats-directory)
      
      (define/public (abort-search)
        (kill-thread search-thread)
        (send search-button enable #t))         
      
      ; PRIVATE
      ; == les catalogues ==
      (define cats (instantiate catalogs% (cats-directory)))
      
      ; le thread utilis lors de la recherche
      (define search-thread #f)
      
      ; INTERFACE UTILISATEUR
      ; pour que 'this' soit instanci...
      (super-instantiate ())
      
      ; on peut maintenant s'occuper du splitter principal
      (send this set-label-position 'vertical)
      (send this spacing 4)
      (send this set-label-position 'vertical)
      (send this set-control-font 
            (make-object font% ui-font-size 'modern 'normal 'normal #f))
      
      ; -  gauche, la liste des catalogues
      (define search-cats-list
        (let ((tcl (instantiate list-box% ()
                     (label "Search &in :")
                     (parent this)
                     (callback void)
                     (min-width 300)
                     (choices '("catalogs list"))
                     (style '(multiple)))))
          (send this attach-left tcl)
          tcl))
      
      ;  droite, le panneau entre/rsultats
      (define search/results-panel
        (let ((srp (instantiate vertical-panel% (this)
                     ;(style '(border))
                     )))
          (send srp set-label-position 'vertical)
          (send this attach-right srp)
          srp))      
      
      ; le remplissage de la liste des rsultats
      (define (display-search-results lr)
        (let ((m-cat (apply max (cons 0 (map (lambda (x) (string-length (car x))) lr))))
              (m-f (min file-string-maxlen
                        (apply max (cons 0 (map (lambda (x) 
                                                  (if (> (length x) 2)
                                                      (string-length (cadr x))
                                                      0)) lr)))))
              (m-s (apply max (cons 0 (map (lambda (x) 
                                             (if (> (length x) 2)
                                                 (string-length (number->string (caddr x)))
                                                 0))
                                           lr)))))
          (search-message "Formatting output...")
          (map (lambda (x)
                 (send search-results append
                       (if (> (length x) 2)
                           (let ((d (seconds->date (list-ref x 3))))
                             (format "~a ~a ~a ~a [~a]" 
                                     (string->ljust-string (format "[~a]" (car x)) (+ m-cat 2))
                                     (string->ljust-string 
                                      (let ((sf (cadr x)))
                                        (if (> (string-length sf) file-string-maxlen)
                                            (string-append (substring sf 0 (- file-string-maxlen 4)) "...")
                                            sf))
                                      m-f)
                                     (string->rjust-string (number->string (caddr x)) m-s) 
                                     (format "~a/~a/~a-~a:~a:~a"
                                             (add-leading-0 (date-day d))
                                             (add-leading-0 (date-month d))
                                             (add-leading-0 (date-year d))
                                             (add-leading-0 (date-hour d)) 
                                             (add-leading-0 (date-minute d))
                                             (add-leading-0 (date-second d)))
                                     (let* ((s (list-ref x 4))
                                            (ls (string-length s)))
                                       (if (< ls 50)
                                           s
                                           (string-append (substring s 0 23)
                                                          "..." 
                                                          (substring s (- ls 23) ls))))))
                           (format "~a* ~a"
                                   (string->ljust-string (format "[~a]" (car x)) (+ m-cat 2))
                                   (cadr x)))
                       x))
               lr)
          (search-message "Done.")))
      
      ; la zone d'entre du critre et le bouton de lancement de la recherche
      (define-values (search-input search-button)
        (let ((p (instantiate horizontal-pane% (search/results-panel)
                   (stretchable-height #f)
                   (alignment '(left bottom))))
              (onclick 
               (lambda (c e)
                 (set! search-thread
                       (thread
                        (lambda ()
                          (let ((preg (regexp 
                                       (let ((s (string-copy 
                                                 (send (send search-input get-editor) get-text))))
                                         (string-uppercase! s)
                                         s)))
                                (l-sel (send search-cats-list get-selections)))
                            (send search-results clear)
                            (send search-button enable #f)
                            (display-search-results
                             (apply append
                                    (map
                                     (lambda(c)
                                       (begin0
                                         (let ((cname (send c get-name)))
                                           (search-message (string-append "Searching " cname))
                                           (map (lambda (x)
                                                  (cons cname x))
                                                (send c catalog-filter-list
                                                      (lambda(x)
                                                        (regexp-match preg 
                                                                      (let ((s (string-copy (car x))))
                                                                        (string-uppercase! s)
                                                                        s))))))
                                         (send c free)))
                                     (map (lambda(i)
                                            (send cats get-catalog-by-index i))
                                          l-sel))))
                            (send search-results append "** End of search **" #f)
                            (search-message #f)
                            (send search-button enable #t))))))))
          (values
           (instantiate text-field% ("Reg&Exp :" p )
             (callback
              (lambda (t e)
                (when (eq? 'text-field-enter (send e get-event-type))
                  (send search-button command
                        (instantiate control-event% () (event-type 'button)))))))
           (instantiate button% ("&Search !" p onclick))
           )))
      
      ; la liste des resultats de la recherche
      (define search-results
        (let ((lb% (class list-box%
                     (override on-subwindow-char)
                     (define (on-subwindow-char l e)
                       (case (send e get-key-code)
                         ((#\return 'numpad-enter)
                          (on-search-result-selected 
                           (send l get-data (send l get-selection))))
                         (else #f)))
                     
                     (super-instantiate ()))))   
          (instantiate lb% ()
            (label "Search &Results :")
            (parent search/results-panel)
            (callback 
             (lambda (l e)
               (when (eq? (send e get-event-type) 'list-box-dclick)
                 (on-search-result-selected 
                  (send l get-data (send l get-selection))))))
            (choices '("** search results **")))))
      ))
  ) ; end module
