(module catview mzscheme
  (require (lib "class.ss")
           ;(lib "string.ss")
           (lib "mred.ss" "mred")
           "utils.scm"
           "db.scm"
           "splitter.scm")
  
  (provide catalog-viewer%)
  
  (define file-string-maxlen 50)
  
  ; bote de dialogue de cration d'un nouveau catalogue
  (define (dialog-new-catalog parent)
    ;
    (define f (let ((ff (make-object dialog% "New catalog" parent 500)))
                (send ff set-label-position 'vertical)
                ff))
    
    (define ok? #f)
    
    (define done 
      (lambda (r) 
        (if r
            (lambda (b e) 
              (if (and (not (string=? (send t get-value) ""))
                       (not (string=? (send t2 get-value) "")))
                  (begin
                    (set! ok? r) 
                    (send f show #f))
                  (begin
                    (message-box "Error..." 
                                 "Catalog name or path is empty !"
                                 f '(ok))
                    (send t focus))))
            (lambda (b e) 
              (set! ok? r) 
              (send f show #f)))))
    ;
    (define t (make-object text-field% 
                "Catalog name :" f 
                (lambda (t e) 
                  (when (eq? (send e get-event-type) 'text-field-enter)
                    ((done #t) #f #f)))
                ""))
    (define p1 (make-object horizontal-pane% f))
    (define t2 (make-object text-field% 
                 "Path to catalog :" p1 
                 (lambda (t e) 
                   (when (eq? (send e get-event-type) 'text-field-enter)
                     ((done #t) #f #f)))
                 ""))
    (define p2 (make-object horizontal-pane% f))
    
    (send p1 set-alignment 'right 'bottom)
    (make-object button% "Browse..." p1 (lambda (b e) 
                                          (let ((r (get-directory "message" f #f '())))
                                            (if r
                                                (send t2 set-value r)))))
    (send p2 set-alignment 'right 'center)
    (send f stretchable-height #f)
    (make-object button% "Cancel" p2 (done #f))
    (make-object button% "    OK    " p2 (done #t) '(border))
    (send (send t get-editor) select-all)
    (send t focus)
    ;(send f center)
    (send f show #t)
    (and ok? `(,(send t get-value)
               ,(let ((p (send t2 get-value)))
                  (dir-slashify p)))))
  
  
  ; le panneau des catalogues, liste des rpertoires et fichiers
  (define catalog-viewer%
    (class vertical-panel%
      ; FIELD
      (init-field 
       cats-directory
       (ui-font-size 12))
      
      ; METHODS
      ; - celles concernant la gestion des catalogues 
      (define/public (set-cats-directory s)
        (set! cats-directory (dir-slashify s))
        (set! cats (instantiate catalogs% (cats-directory)))
        (update-cats-display #f))
      
      (define/public (get-cats-directory) cats-directory)
      
      (define/public (get-current-cat-name)
        (send current-cat get-name))
      
      ; slectionner le fichier/rpertoire associ aux donnes contenues dans l
      ; - si rpertoire -> l == (catalogue chemin)
      ; - sinon -> l == (catalogue nom_fichier taille date chemin)
      (define/public (select-data l)
        (when l
          (do ((n 0 (+ n 1)))
            ((string=? (car l) (send cb-catalogs get-string n))
             (when (not (string=? (car l) (send cb-catalogs get-string-selection)))
               (send* cb-catalogs 
                 (set-selection n)
                 (command (instantiate control-event% () (event-type 'choice)))))
             (let ((dir (list-ref l (if (> (length l) 2) 4 1))))
               (do ((n 0 (+ n 1)))
                 ((string=? (send dir-lb get-data n) dir)
                  (send* dir-lb 
                    (set-selection n)
                    (command (instantiate control-event% () (event-type 'list-box)))))))
             (when (> (length l) 2)
               (let ((f (list-ref l 1)))
                 (do ((n 0 (+ n 1)))
                   ((string=? (send files-lb get-data n) f)
                    (send* files-lb 
                      (set-selection n)
                      ;(command (instantiate control-event% () (event-type 'list-box)))
                      )))))))))
      
      ; pour la gestion spciale du clavier dans l'arbre d'affichae des rpertoires
      (define/public (frame-process-char? target event)
        (not (and (eq? target dir-lb)
                  (send event get-control-down)
                  (memq (send event get-key-code) '(left right)))))
      
      (define/public (abort-add-catalog)
        (kill-thread thread-add-cat)
        (send btn-add-catalog enable #t))
      
      ; CALLBACKS
      ;  surcharger pour savoir ce qui se passe dans le contrle
      ; l'utilisateur a chang de catalogue dans la liste droulante
      (define/public (on-cat-changed) #t)
      
      ; ajout ou suppression de catalogue effectu
      (define/public (on-cat-added/deleted) #t) 
      
      ; info sur le droulement d'une opration, d'une slection de rp/fichier
      (define/public (on-process-message s) #t)
      
      ; progression du processus d'ajout de catalogue
      (define/public (on-add-catalog-progress s) #t)
      
      ; PRIVATE
      ; == les catalogues ==
      (define cats (instantiate catalogs% (cats-directory)))
      
      ; le catalogue courant
      (define current-cat #f)
      
      ; le thread de cration d'un catalogue
      (define thread-add-cat #f)
      
      ; mise  jour de la liste des catalogues, des rpertoires,
      ; et initialisation du catalogue courant
      (define (update-cats-display curr-cat-name)
        (send cb-catalogs clear) 
        (clear-dir/file-lb)
        (set! current-cat #f)
        (when (not (null? (send cats get-catalogs)))
          (map (lambda (x)
                 (send cb-catalogs append x))
               (map (lambda (c)
                      (send c get-name))
                    (send cats get-catalogs)))
          (if curr-cat-name
              (let ((c (or (send cats get-catalog-by-name curr-cat-name)
                           (car (send cats get-catalogs)))))
                (set! current-cat c)
                (send cb-catalogs set-string-selection curr-cat-name))
              (set! current-cat
                    (car (send cats get-catalogs))))
          (fill-hlist)
          (on-process-message ""))) 
      
      ; la fonction de remplissage de l'arbre des rpertoires
      (define (fill-hlist)
        ;
        (define (fill-node tree pref last?)
          (if (null? (cdr tree))
              (send dir-lb append 
                    (string-append pref (if last? "'- " "- ") (caar tree))
                    (cadar tree))
              (begin
                (send dir-lb append (string-append pref (if last? "'- " "-- ") (caar tree)) (cadar tree))
                (let loop ((x (cdr tree)))
                  (when (not (null? x))
                    (fill-node (car x) 
                               (string-append pref 
                                              (if (null? (cdr x))" " " |"))
                               (null? (cdr x)))
                    (loop (cdr x)))))))
        ;
        (fill-node (send* current-cat
                     (read-catalog) 
                     (catalog-dirtree))
                   " " #f))
      
      (define (clear-dir/file-lb)
        (send* dir-lb (set '()) (refresh))
        (send* files-lb (set '()) (refresh))    
        (yield))
      
      ; pour pouvoir utiliser this
      (super-instantiate ())
      
      ; INTERFACE 
      (send this set-control-font 
            (make-object font% ui-font-size 'modern 'normal 'normal #f))
      
      ; le bandeau de commandes sur les catalogues
      (define cats-cmd-panel
        (instantiate horizontal-panel% (this)
          (stretchable-height #f)))
      
      ; la liste droulante des catalogues disponibles
      (define cb-catalogs
        (instantiate choice% ()
          (parent cats-cmd-panel)
          (label "Catalo&gs :")
          (stretchable-height #f)
          (stretchable-width #t)
          (choices '())
          (callback (lambda (cb evt)
                      (send cb refresh)
                      (clear-dir/file-lb)
                      (set! current-cat (list-ref (send cats get-catalogs)
                                                  (send cb get-selection)))
                      (fill-hlist)
                      (on-cat-changed)))))
      
      ; le bouton d'ajout de catalogue
      (define btn-add-catalog
        (instantiate button% ("&New catalog ..." cats-cmd-panel)
          (callback 
           (lambda (b evt)
             (let ((r (dialog-new-catalog (send this get-top-level-window))))
               (when r
                 (send btn-add-catalog enable #f)
                 (set! thread-add-cat
                       (thread (lambda ()
                                 (let ((c (send cats add-catalog (car r) (cadr r) 
                                                (lambda (s)
                                                  (on-add-catalog-progress s)
                                                  (on-process-message s)))))
                                   (on-add-catalog-progress "Saving catalog...")
                                   (send c save-catalog)
                                   (send cats init-catalogs)
                                   (on-add-catalog-progress #f)
                                   (send btn-add-catalog enable #t)
                                   (update-cats-display (car r))
                                   (send dir-lb focus)
                                   (on-cat-added/deleted)))))))))))
      
      ; le bouton de suppression du catalogue
      (define btn-delete-catalog
        (instantiate button% ("&Delete catalog ..." cats-cmd-panel)
          (callback (lambda(b evt)
                      (when (and current-cat
                                 (eq? 'ok
                                      (message-box "Delete catalog" 
                                                   "Caution : this operation can't be canceled.\nProceed ?"
                                                   (send this get-top-level-window)
                                                   '(ok-cancel))))
                        (send* cats 
                          (delete-catalog current-cat)
                          (init-catalogs))
                        (update-cats-display #f)
                        (on-cat-added/deleted))))))
      
      ; 2- le panneau dirs/files, avec dfinition des fontes des contrles  
      (define dir/files-splitter 
        (let ((df (instantiate vertical-splitter% (this) 
                    (spacing 4))))
          (send df set-label-position 'vertical)
          df))
      
      ; la classe de la liste des rpertoires, avec gestion spciale du clavier
      (define dir-lb%
        (class list-box%
          (override on-subwindow-char)
          
          (define on-subwindow-char
            (lambda (receiver event)
              (if (send event get-control-down)
                  (case (send event get-key-code)
                    ((up)
                     (let* ((n (send this get-selection))
                            (p (and n (send this get-data n))))
                       (when p
                         (let-values (((b ?- ?--) (split-path p)))
                           (let loop ((nn (- n 1)))
                             (if (< nn 0)
                                 #t
                                 (let-values (((bb ?- ?--) (split-path (send this get-data nn))))
                                   (if (or (eq? b bb) 
                                           (and (string? b)
                                                (string? bb)
                                                (string=? b bb)))
                                       (begin
                                         (send this select nn)
                                         (send this command (instantiate control-event% () (event-type 'list-box)))
                                         #t)
                                       (loop (- nn 1))))))))))
                    ((down)
                     (let* ((n (send this get-selection))
                            (p (and n (send this get-data n))))
                       (when p
                         (let-values (((b ?- ?--) (split-path p)))
                           (let loop ((nn (+ n 1)))
                             (if (= nn (send this get-number))
                                 #t
                                 (let-values (((bb ?- ?--) (split-path (send this get-data nn))))
                                   (if (or (eq? b bb) 
                                           (and (string? b)
                                                (string? bb)
                                                (string=? b bb)))
                                       (begin
                                         (send this select nn)
                                         (send this command (instantiate control-event% () (event-type 'list-box)))
                                         #t)
                                       (loop (+ nn 1))))))))))
                    ((left)
                     (let* ((n (send this get-selection))
                            (p (and n (send this get-data n))))
                       (when p
                         (let-values (((b ?- ?--) (split-path p)))
                           (send this select 
                                 (let loop ((nn (- n 1)))
                                   (if (or (<= nn 0) (not b))
                                       0
                                       (let-values (((bb ?- ?--) (split-path (send this get-data nn))))
                                         (if (or (eq? b bb) 
                                                 (and (string? b)
                                                      (string? bb)
                                                      (string<? bb b)))
                                             nn
                                             (loop (- nn 1)))))))
                           (send this command (instantiate control-event% () (event-type 'list-box)))
                           #t))))
                    ((right)
                     (let* ((n (send this get-selection))
                            (m (and n (length (path-chain (send this get-data n))))))
                       (when m
                         (send this select 
                               (let loop ((nn (+ n 1)))
                                 (if (= nn (send this get-number))
                                     n
                                     (let ((mm (length (path-chain (send this get-data nn)))))
                                       (cond
                                         ((< mm m) n)
                                         ((>= mm m) nn)
                                         (else (loop (+ nn 1)))))))) 
                         (send this command (instantiate control-event% () (event-type 'list-box)))
                         #t)))
                    (else #f))
                  #f)))
          
          (super-instantiate ())))
      
      ; la liste des rpertoires
      (define dir-lb
        (letrec 
            ((on-dir-select 
              (lambda (l evt)
                (let* ((s (send dir-lb get-data (send dir-lb get-selection)))
                       (l-files (send current-cat list-files/dir s))
                       (m1 (min file-string-maxlen 
                                (apply max 
                                       (cons 0 
                                             (map (lambda (x) (string-length (car x))) 
                                                  l-files)))))
                       (m2 (apply max 
                                  (cons 0 
                                        (map (lambda (x) (string-length (number->string (cadr x))))
                                             l-files)))))
                  (on-process-message s)
                  (send files-lb clear)
                  (for-each
                   (lambda (x)
                     (send files-lb append
                           (let ((d (seconds->date (caddr x))))
                             (format "~a~a ~a" 
                                     (string->ljust-string 
                                      (let ((sf (car x)))
                                        (if (> (string-length sf) file-string-maxlen)
                                            (string-append (substring sf 0 (- file-string-maxlen 4)) "...")
                                            sf))
                                      (+ m1 1))
                                     (string->rjust-string (number->string (cadr x)) (+ m2 1)) 
                                     (format "~a/~a/~a-~a:~a:~a"
                                             (add-leading-0 (date-day d))
                                             (add-leading-0 (date-month d))
                                             (add-leading-0 (date-year d))
                                             (add-leading-0 (date-hour d)) 
                                             (add-leading-0 (date-minute d))
                                             (add-leading-0 (date-second d)))))
                           (car x)))
                   l-files))
                #t))
             (d (instantiate dir-lb% ()
                  (label "Catalog &Tree :") 
                  (choices '())
                  (parent dir/files-splitter)
                  (callback on-dir-select)
                  (min-width 300))))
          (send dir/files-splitter attach-left d)
          d))
      
      ; la liste des fichiers
      (define files-lb 
        (let ((f (instantiate list-box% ()
                   (label "Directory Fi&les :") 
                   (choices '("fichiers..."))
                   (parent dir/files-splitter)
                   (callback (lambda(l evt) 
                               (on-process-message 
                                (send files-lb get-data 
                                      (send files-lb get-selection)))))
                   (min-width 100))))
          (send dir/files-splitter attach-right f)
          f))
      
      (update-cats-display #f)
      )))
