(module db mzscheme
  (require (lib "class.ss")
           (lib "pretty.ss")
           ;(lib "string.ss")
           (lib "list.ss")
           )
  
  (provide set-pretty-print-catalog! catalogs% path-chain)
  
  ;; Essai de catalogue de CD/rpertoires
  
  (define version-tag '(catalog 0 1))
  
  ;;; retourne la liste des composants d'un chemin d'accs  un fichier
  ;;; (path-chain "c:/a/b/c.txt") => ("c:/" "a" "b" "c.txt")
  (define (path-chain p)
    (define (loop p l)
      (let-values (((p1 p2 p3) (split-path p)))
        (if (not p1)
            (cons p2 l)
            (loop p1 (cons p2 l)))))
    (loop p '()))

  (define pretty-print-catalog #f)
  
  (define (set-pretty-print-catalog! v)
    (set! pretty-print-catalog v))
  
  ;;; Classes externes
  ;;; Reprsente l'ensemble des catalogues contenus dans un rpertoire 
  (define catalogs%
    (class object%
     
      ;; Cette classe interne reprsente un catalogue.
      ;; Son interface est utilisable en dehors de ce module car des objets de type catalog%
      ;; sont retourns par les mthodes publiques de catalogs% (au pluriel !)
      ;; en bref, il n'est possible d'obtenir des objets catalog% qu' travers les mthodes de catalogs%
      (define catalog%
        (class object%
          ;; champs
          (init-field cat-name)
          
          ;;; variables prives
          ;; la liste hirarchique reprsentant le catalogue
          (define l-cat null)
          
          ;; le catalogue doit-il tre sauv ?
          (define must-save? #f)
          
          ;;; Mthodes
          (define/public (free)
            (when (and l-cat must-save?)
              (save-catalog)
              (set! must-save? #f))
            (set! l-cat null))
          
          (define/public (get-name) cat-name)
          ;
          (define/public (get-catalog-list) l-cat)    
          ;
          ;; la fonction de catalogage.
          ;; <= : chemin  cataloguer (string) 
          ;; => : arbre des rp/fichiers (liste) 
          ;; arbre dans l-cat
          (define/public (make-catalog path callback)
            ;
            (define cbk (or callback (lambda (d) #f)))
            
            (define (make-files/dir-list path l-files)
              (if (null? l-files)
                  '()
                  (let* ((f (car l-files))
                         (ff (string-append path f)))
                    (if (link-exists? ff)
                        (make-files/dir-list path (cdr l-files))
                        (with-handlers ((exn:i/o:filesystem?
                                         (lambda (exn) 
                                           (make-files/dir-list path (cdr l-files)))))
                          (cons (if (directory-exists? ff)
                                    (begin
                                      (cbk ff)
                                      `(dir ,ff 
                                            ,@(make-files/dir-list (string-append ff "/")
                                                                   (directory-list ff))))
                                    `(,f ,(file-size ff) ,(file-or-directory-modify-seconds ff)))
                                (make-files/dir-list path (cdr l-files))))))))
            ;
            (set! l-cat
                  `(,version-tag ,cat-name
                    (dir ,path ,@(make-files/dir-list path (directory-list path)))))
            (set! must-save? #t)
            l-cat)
          ;
          (define/public (save-catalog)
            (let ((db-filename (make-db-filename cat-name)))
              (if (file-exists? db-filename)
                  (delete-file db-filename))
              
              (with-output-to-file db-filename
                (lambda ()
                  (if pretty-print-catalog
                      (pretty-print l-cat)
                      (write l-cat))
                  ))
              (set! must-save? #f)))
          ;
          ;; sauvegarde du catalogue dans un fichier
          ;; (string)(string) => void
          (define/public (make/save-catalog path callback)
            (make-catalog cat-name path callback)
            (save-catalog))
          
          ;; lecture d'un catalogue
          (define/public (read-catalog)
            (when (null? l-cat)
              (set! l-cat
                    (with-input-from-file (make-db-filename cat-name) 
                      (lambda () (read)))))
            (set! must-save? #f)
            l-cat)
          
          (define/public (catalog-filter-list pred?)
            ;
            (define (explore-cat cat path-in-catalog)
              ;; (display (format "~a ~a ~n" cat path-in-catalog))
              (cond
                ((null? cat) '())
                ((eq? (car cat) 'dir)
                 (let-values (((? name ??) (split-path (cadr cat))))
                   (if (pred? (list name))
                       (cons (list (cadr cat))
                             (explore-cat (cddr cat) (cadr cat)))
                       (explore-cat (cddr cat) (cadr cat)))))
                ((eq? (caar cat) 'dir)
                 (append
                  (explore-cat (car cat) path-in-catalog)
                  (explore-cat (cdr cat) path-in-catalog)))
                (else
                 (if (pred? (car cat))
                     (cons (append (car cat) (list path-in-catalog))
                           (explore-cat (cdr cat) path-in-catalog))
                     (explore-cat (cdr cat) path-in-catalog)))))
            ;
            (read-catalog)
            (explore-cat (caddr l-cat) ""))
          
          ;; retourne la liste des fichiers contenus dans un catalogue
          (define/public (catalog-filelist)
            (catalog-filter-list (lambda(x) #t)))
          
          ; retourne l'arbre des rpertoires du catalogue
          ; chaque rpertoire est de la forme (nom_rpertoire_sans_chemin nom_avec_chemin)
          ; l'arbre est de la forme (<rp> <sous-rp_1> <sous-rp_2> ... ) 
          (define/public (catalog-dirtree)
            (read-catalog)
            (let loop ((l (caddr l-cat)))
              (let ((ll (filter (lambda (x) (eq? (car x) 'dir)) (cddr l))))
                `((,(car (last-pair (path-chain (cadr l)))) ,(cadr l))
                  ,@(map loop ll)))))
          
          ; retourne la liste des fichiers pour un rpertoire
          (define/public (list-files/dir r)
            ; pour savoir si s1 est au dbut de s2
            ; retourne deux valeurs : 
            ; - si s2 dbute par s1
            ; - si s1=s2
            (define (begin-with s1 s2)
              (define n1 (string-length s1))
              (define n2 (string-length s2))
              ;
              (define (loop n)
                (cond
                  ((= n n1) #t)
                  ((eqv? (string-ref s1 n) (string-ref s2 n))
                   (loop (+ n 1)))
                  (else #f)))
              ;
              (values
               (and (>= n2 n1) (loop 0))
               (= n1 n2)))
            ;
            (define (find-dir tree)
              (let-values (((ancestor? found?) (begin-with (cadr tree) r)))
                (if ancestor?
                    (if found?
                        tree
                        (let loop ((l (cddr tree)))
                          (if (null? l)
                              #f
                              (let ((x (car l)))
                                (if (eq? 'dir (car x))
                                    (or (find-dir x)
                                        (loop (cdr l)))
                                    (loop (cdr l)))))))
                    #f)))
            
            (read-catalog)
            (filter (lambda (x) (not (eq? (car x) 'dir)))
                    (cddr (find-dir (caddr l-cat)))))
          
          ; regroupe dans une a-liste indexe par leur taille les fichiers d'un catalogue.
          (define/public group-by-size 
            (let ((gbs (lambda (l-files)
                         (let loop ((r '())
                                    (l l-files))
                           (if (null? l)
                               r
                               (let* ((f (car l))
                                      (b (if (null? (cdr f))
                                             #f
                                             (assv (cadr f) r))))
                                 (if b
                                     (begin
                                       (set-cdr! b (cons f (cdr b)))
                                       (loop r (cdr l)))
                                     (loop (if (null? (cdr f))
                                               r
                                               (cons (list (cadr f) f) r))
                                           (cdr l)))))))))
              (case-lambda
                (() (gbs (catalog-filelist)))
                ((pred?) (gbs (catalog-filter-list pred?))))))
          
          
          (super-instantiate ())
          (set! must-save? #f))
        )
      
      ;==== CATALOGS ====
      ;;; Champs
      ;; le nom du rpertoire des catalogues
      (init-field catalogs-path)
      
      ;;; Variables prives
      ;; la liste des catalogues
      (define l-cat '())
      
      ;; pour retrouver les fichiers catalogues  partir de leur extension
      (define preg (regexp "\\.cat$"))
      
      (define (l-catalog-names)
        (if (directory-exists? catalogs-path)
            (let loop ((ld (directory-list catalogs-path))
                       (l-f '()))
              (if (null? ld)
                  (quicksort l-f string<?)
                  (loop (cdr ld)
                        (let ((fname (car ld)))
                          (if (and (regexp-match preg fname)
                                   (not (directory-exists? 
                                         (string-append catalogs-path fname))))
                              (cons (substring fname 0 (- (string-length fname) 4)) 
                                    l-f)
                              l-f)))))
            (begin
              (make-directory catalogs-path)
              '())))
      
      (define (make-db-filename cat-name)
        (string-append catalogs-path cat-name  ".cat"))
      
      ;;; Mthodes
      (define/public (add-catalog cat-name cat-path callback)
        (let ((c (instantiate catalog% (cat-name))))
          (send c make-catalog cat-path callback)
          c))
      
      (define/public (delete-catalog cat)
        (delete-file (make-db-filename (send cat get-name)))
        (init-catalogs))
      
      (define/public (get-catalogs)
        l-cat)
      
      (define/public (get-catalog-by-name name)
        (let loop ((l l-cat))
          (if (null? l)
              #f
              (if (string=? name (send (car l) get-name))
                  (car l)
                  (loop (cdr l))))))
      
      (define/public (get-catalog-by-index i)
        (list-ref l-cat i))
      
      
      (define/public (init-catalogs)
        (set! l-cat
              (map
               (lambda (n)
                 (instantiate catalog% (n)))
               (l-catalog-names))))  
      
      ;;; Initialisation
      (super-instantiate ())
      (init-catalogs)
      )))